# -*- coding: utf8 -*-
import json
import urllib2
import logging

logger = logging.getLogger()
logger.setLevel(logging.INFO)

# ES内网地址，填写格式如：http://10.0.3.14:9200
es_endpoint = ""
# ES访问密码，如果不是白金版则忽略
es_password = ""

# 样例数据索引名，默认为es_corpus_0126，请确保该索引没有在业务中使用，可保持默认值
es_index = "es_corpus_0126"


def main_handler(event, context):
    logger.info(event)

    if 'upload' in event['path']:
        return delete_create_upload()

    if 'q' not in event['queryString']:
        f = open("./index.html")
        html = f.read()
        return {
            "isBase64Encoded": False,
            "statusCode": 200,
            "headers": {'Content-Type': 'text/html'},
            "body": html
        }

    return search(event['queryString']['q'])


def make_rsp(body):
    return {
        "headers": {
            "Content-Type": "application/json",
            "Access-Control-Allow-Origin": "*"
        },
        "isBase64Encoded": False,
        "statusCode": 200,
        "body": json.dumps(body, ensure_ascii=False),
    }


def get_words(filename):
    with open("./" + filename, "r") as f:
        lines = f.readlines()
        return [line for line in lines if line.strip() != ""]


def delete_create_upload():
    password_manager = urllib2.HTTPPasswordMgrWithDefaultRealm()
    password_manager.add_password(None, es_endpoint, 'elastic', es_password)
    auth_manager = urllib2.HTTPBasicAuthHandler(password_manager)
    opener = urllib2.build_opener(auth_manager)
    urllib2.install_opener(opener)

    req = urllib2.Request("{}/{}".format(es_endpoint, es_index))
    method = "DELETE"
    req.get_method = lambda: method
    try:
        rsp = urllib2.urlopen(req)
    except:
        pass
    logger.info("DELETE DONE")

    synonyms = get_words("synonym.dic")
    stopwords = get_words("stopword.dic")

    data = {
        "settings": {
            "number_of_shards": 1,
            "number_of_replicas": 1,
            "refresh_interval": "1s",
            "analysis": {
                "analyzer": {
                    "my_analyzer": {
                        "type": "custom",
                        "tokenizer": "ik_smart",
                        "filter": ["my_synonym", "my_stop"]
                    }
                },
                "filter": {
                    "my_synonym": {
                        "type": "synonym",
                        "lenient": True,
                        "synonyms": synonyms
                    },
                    "my_stop": {
                        "type": "stop",
                        "lenient": True,
                        "stopwords": stopwords
                    }
                }
            }
        },
        "mappings": {
            "_doc": {
                "properties": {
                    "title": {
                        "type": "text",
                        "analyzer": "my_analyzer",
                        "fielddata": False
                    },
                    "url": {
                        "type": "keyword",
                        "doc_values": False
                    },
                    "content": {
                        "type": "text",
                        "analyzer": "my_analyzer",
                        "fielddata": False
                    }
                }
            }
        }
    }
    req = urllib2.Request("{}/{}".format(es_endpoint, es_index))
    req.add_header('Content-Type', 'application/json')
    req.add_data(json.dumps(data))
    method = "PUT"
    req.get_method = lambda: method
    create_ok = False
    try:
        rsp = urllib2.urlopen(req)
    except urllib2.HTTPError, e:
        logger.info("create code is {}".format(e.getcode()))
        if e.getcode() == 401:
            return make_rsp({"msg": "你购买的是白金版的集群哦，需要设置es_password"})
    except ValueError, e:
        return make_rsp({"msg": "检查下你的ES内网地址哦"})
    except Exception, e:
        logger.error(e)
    else:
        result = json.loads(rsp.read(), encoding='utf-8')
        logger.info(result)
        if result['acknowledged']:
            create_ok = True

    if create_ok:
        logger.info("CREATE DONE")
        return upload()

    return make_rsp({"msg": "样例数据导入失败，检查一下es_endpoint和es_password哦"})


def upload():
    password_manager = urllib2.HTTPPasswordMgrWithDefaultRealm()
    password_manager.add_password(None, es_endpoint, 'elastic', es_password)
    auth_manager = urllib2.HTTPBasicAuthHandler(password_manager)
    opener = urllib2.build_opener(auth_manager)
    urllib2.install_opener(opener)

    data = []
    with open("./data.json", "r") as f:
        lines = f.readlines()
        for line in lines:
            data.append(json.loads(line))
    encoded_data = "\n".join(map(json.dumps, data)) + "\n"
    req = urllib2.Request("{}/{}/_doc/_bulk".format(es_endpoint, es_index), data=encoded_data)
    req.add_header('Content-Type', 'application/json')
    method = "POST"
    req.get_method = lambda: method
    rsp = urllib2.urlopen(req)
    logger.info("UPLOAD DONE")
    return make_rsp({"msg": "样例数据导入成功，试试搜索吧"})


def search(query):
    password_manager = urllib2.HTTPPasswordMgrWithDefaultRealm()
    password_manager.add_password(None, es_endpoint, 'elastic', es_password)
    auth_manager = urllib2.HTTPBasicAuthHandler(password_manager)
    opener = urllib2.build_opener(auth_manager)
    urllib2.install_opener(opener)

    data = {
        "size": "3",
        "query": {
            "match": {
                "content": query
            }
        }
    }
    req = urllib2.Request("{}/{}/_search".format(es_endpoint, es_index), json.dumps(data))
    req.add_header('Content-Type', 'application/json')
    rsp = urllib2.urlopen(req)
    if rsp.getcode() == 200:
        result = json.loads(rsp.read(), encoding='utf-8')
        count = len(result["hits"]["hits"])
        answers = []
        if count > 0:
            if count > 3:
                count = 3
            for i in range(0, count):
                doc = result["hits"]["hits"][i]["_source"]
                answers.append({
                    "title": doc["title"],
                    "content": doc["content"][:200] + "...",
                    "url": doc["url"],
                })
            return make_rsp({"answers": answers, "error": "", "total": count})
        return make_rsp({"answers": [], "error": "我没有找到答案哦，你可以换一种问法~"})
    return make_rsp({"answers": [], "error": "出错了哦~"})
